@@ -1,3 +1,4 @@
+import logging
 from typing import List, Union, Optional, Tuple
 
 from bs4 import BeautifulSoup
@@ -13,12 +14,24 @@ from fuo_bilibili.api.schema.responses import SearchResponse, SearchResultVideo,
     FavoriteSeasonResourceResponse, HistoryLaterVideoResponse, HomeDynamicVideoResponse, UserInfoResponse, \
     UserBestVideoResponse, UserVideoResponse, AudioFavoriteSongsResponse, AudioFavoriteListResponse, AudioPlaylist, \
     AudioPlaylistSong, VideoHotCommentsResponse, SearchResultUser, LiveFeedListResponse, SearchResultLiveRoom, \
-    SearchResultMedia, MediaGetListResponse, VideoWeeklyListResponse
+    SearchResultMedia, MediaGetListResponse, VideoWeeklyListResponse, VideoMostPopularResponse, \
+    HomeRecommendVideosResponse
 from fuo_bilibili.util import format_timedelta_to_hms
 
+logger = logging.getLogger(__name__)
 PROVIDER_ID = __identifier__
 
 
+def get_text_from_html(html):
+    return BeautifulSoup(html, features="html.parser").get_text()
+
+
+def wrap_pic(pic):
+    if pic.startswith('//'):
+        return f'http:{pic}'
+    return pic
+
+
 class BBriefAlbumModel(BriefAlbumModel):
     cover: str = ''
 
@@ -108,7 +121,7 @@ class BSongModel(SongModel):
                 title=page.part,
                 artists=[],
                 duration=page.duration.total_seconds() * 1000,
-                pic_url=result.pic,
+                pic_url=wrap_pic(result.pic),
             )
             children.append(song)
 
@@ -130,7 +143,7 @@ class BSongModel(SongModel):
             )],
             duration=result.duration.total_seconds() * 1000,
             lyric=lrc or '',
-            pic_url=result.pic,
+            pic_url=wrap_pic(result.pic),
             children=children,
         )
 
@@ -211,7 +224,7 @@ class BSearchModel:
         return BVideoModel(
             source=PROVIDER_ID,
             identifier=f'live_{live.roomid}',
-            title=live.title,
+            title=get_text_from_html(live.title),
             artists=[
                 BriefArtistModel(
                     source=PROVIDER_ID,
@@ -229,6 +242,7 @@ class BSearchModel:
         artists = []
         videos = []
         albums = []
+        invalid_count = 0
         if isinstance(response, Tuple):
             results = []
             for r in response:
@@ -241,12 +255,18 @@ class BSearchModel:
         for result in results:
             if isinstance(result, SearchResultVideo):
                 songs.append(BSongModel.create_model(result))
+                videos.append(BVideoModel.create_video_model(result))
             elif isinstance(result, SearchResultUser):
                 artists.append(cls.search_user_model(result))
             elif isinstance(result, SearchResultLiveRoom):
                 videos.append(cls.search_live_model(result))
             elif isinstance(result, SearchResultMedia):
                 albums.append(cls.search_media_model(result))
+            else:
+                invalid_count += 1
+        if invalid_count != 0:
+            logger.warning(f'invalid object in search result: {invalid_count}')
+
         return SimpleSearchResult(
             q=request[0].keyword if isinstance(request, Tuple) else request.keyword,
             songs=songs or [],
@@ -466,7 +486,7 @@ class BVideoModel(VideoModel):
         return cls(
             source=PROVIDER_ID,
             identifier=f'live_{live.roomid}',
-            title=live.title,
+            title=get_text_from_html(live.title),
             artists=[
                 BriefArtistModel(
                     source=PROVIDER_ID,
@@ -477,3 +497,55 @@ class BVideoModel(VideoModel):
             duration=0,
             cover=live.cover,
         )
+
+    @classmethod
+    def create_recommend_video_model(cls, result: HomeRecommendVideosResponse.HomeRecommendVideosResponseData.Video):
+        return VideoModel(
+            source=PROVIDER_ID,
+            identifier=result.bvid,
+            title=get_text_from_html(result.title),
+            artists=[BriefArtistModel(
+                source=PROVIDER_ID,
+                identifier=result.owner.mid,
+                name=result.owner.name
+            )],
+            duration=result.duration.total_seconds(),
+            cover=wrap_pic(result.pic),
+            play_count=result.stat.view,
+        )
+
+
+    @classmethod
+    def create_popular_video_model(cls, result: VideoMostPopularResponse.VideoMostPopularResponseData.PopularVideo):
+        return VideoModel(
+            source=PROVIDER_ID,
+            identifier=result.bvid,
+            title=get_text_from_html(result.title),
+            artists=[BriefArtistModel(
+                source=PROVIDER_ID,
+                identifier=result.owner.mid,
+                name=result.owner.name
+            )],
+            duration=result.duration.total_seconds(),
+            cover=wrap_pic(result.pic),
+            play_count=result.stat.view,
+            released=result.pubdate.strftime('%Y-%m-%d'),
+        )
+
+    @classmethod
+    def create_video_model(cls, result: SearchResultVideo) -> 'VideoModel':
+        return VideoModel(
+            source=__identifier__,
+            identifier=result.bvid,
+            title=get_text_from_html(result.title),
+            artists=[BriefArtistModel(
+                source=__identifier__,
+                name=result.author,
+                identifier=result.mid
+            )],
+            duration=result.duration.total_seconds(),
+            cover=wrap_pic(result.pic),
+            play_count=result.play,
+            released=result.pubdate.strftime('%Y-%m-%d'),
+        )
+
